# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

''' Utility methods related to Koji '''

from __future__ import absolute_import
import os

import collections
import koji
import hawkey

from libtaskotron import file_utils
from libtaskotron.logger import log
from libtaskotron import exceptions as exc
from libtaskotron import rpm_utils
from libtaskotron import config


class KojiClient(object):
    '''Helper Koji methods.

    :ivar koji.ClientSession session: Koji client session
    '''

    def __init__(self, koji_session=None):
        '''Create a new KojiClient

        :param koji_session: an existing Koji session instance or ``None`` if
                             you want a new default session to be created
        :type koji_session: :class:`koji.ClientSession`
        '''
        self.session = (koji_session or
            koji.ClientSession(config.get_config().koji_url))

        # contains a rpm_filename -> build info mapping used in rpm_to_build
        self._rpm_to_build_cache = {}


    def rpm_to_build(self, rpm, prefetch=True):
        '''Get koji build object for the rpm.

        :param str rpm: filename as either ``/my/path/nvr.a.rpm`` or ``nvr.a.rpm``
        :param bool prefetch: if set to True, get list of all rpms for the build
           and store it for future use - this will speed up subsequent queries
           for rpms belonging to already queried build.
        :return: Koji buildinfo dictionary (as returned e.g. from :meth:`koji.getBuild`)
        :rtype: :class:`bunch.Bunch`
        :raise TaskotronRemoteError: if rpm or it's related build is not found
        '''

        # extract filename from the (possible) path
        rpm = os.path.split(rpm)[-1]

        if rpm in self._rpm_to_build_cache:
            return self._rpm_to_build_cache[rpm]

        rpminfo = self.session.getRPM(rpm)
        if rpminfo is None:
            raise exc.TaskotronRemoteError('RPM not found')

        buildinfo = self.session.getBuild(rpminfo['build_id'])
        if buildinfo is None:
            raise exc.TaskotronRemoteError('Build %r not found' % rpminfo['build_id'])

        if prefetch:
            rpms = self.session.listRPMs(rpminfo['build_id'])
            for r in rpms:
                filename = "%s.%s.rpm" % (r['nvr'], r['arch'])
                self._rpm_to_build_cache[filename] = buildinfo
        else:
            self._rpm_to_build_cache[rpm] = buildinfo

        return buildinfo


    def nvr_to_urls(self, nvr, arches=None, debuginfo=False, src=True):
        '''Get list of URLs for RPMs corresponding to a build.

        :param str nvr: build NVR
        :param arches: restrict the arches of builds to provide URLs for. Note:
                       If basearch ``i386`` is in the list, ``i686`` arch is
                       automatically added (since that's the arch Koji API uses).
        '''
        log.info('Querying Koji for a list of RPMS for: %s', nvr)

        # add i686 arch if i386 is present in arches
        if arches and 'i386' in arches and 'i686' not in arches:
            arches.append('i686')

        info = self.session.getBuild(nvr)
        if info is None:
            raise exc.TaskotronRemoteError(
                "No build information found for %s" % nvr)

        baseurl = '/'.join((config.get_config().pkg_url, info['package_name'],
                            info['version'], info['release']))

        rpms = self.session.listRPMs(buildID=info['id'], arches=arches)
        if not debuginfo:
            rpms = [r for r in rpms if not r['name'].endswith('-debuginfo')]
        if not src:
            rpms = [r for r in rpms if not r['arch'] == 'src']

        urls = ['%s/%s' % (baseurl, koji.pathinfo.rpm(r)) for r in rpms]
        if len(urls) == 0:
            raise exc.TaskotronRemoteError('No RPMs found for %s' % nvr)

        return sorted(urls)


    def get_nvr_rpms(self, nvr, rpm_dir, arches=None, debuginfo=False,
                     src=False):
        '''Retrieve the RPMs associated with a build NVR into the specified
            directory.

        :param str nvr build: NVR
        :param str rpm_dir: location where to store the RPMs
        :return: list of local filenames of the grabbed RPMs
        :rtype: list of str
        :raise TaskotronRemoteError: if issues with target directory
        '''

        try:
            rpm_urls = self.nvr_to_urls(nvr,
                                        arches=arches,
                                        debuginfo=debuginfo,
                                        src=src)
        except exc.TaskotronRemoteError, e:
            log.warn('Error while retrieving urls for %s (%s)' % (nvr, arches))
            log.warn(e)
            return

        rpm_files = []
        log.info('Fetching %s RPMs for: %s', len(rpm_urls), nvr)
        for url in rpm_urls:
            log.debug('  Fetching %s', url)
            rpm_file = file_utils.download(url, rpm_dir)
            rpm_files.append(rpm_file)

        return rpm_files


    def get_tagged_rpms(self, tag, dest, arches):
        '''Downloads all RPMs of all NVRs tagged by a specific Koji tag.

        :param str tag: Koji tag to be queried for available builds, e.g.
                        ``f20-updates-pending``
        :param str dest: destination directory
        :param arches: list of architectures
        :type arches: list of str
        :return: list of local filenames of the grabbed RPMs
        :rtype: list of str
        :raise TaskotronRemoteError: if issues with target directory
        '''
        log.debug('Querying Koji for tag: %s' % tag)
        tag_data = self.session.listTagged(tag)

        nvrs = ["%(nvr)s" % x for x in tag_data]
        rpms = []
        log.info("Fetching %s builds for tag: %s", len(nvrs), tag)
        log.debug('Builds to be downloaded:\n  %s', '\n  '.join(nvrs))

        for nvr in nvrs:
            rpms.append(self.get_nvr_rpms(nvr, dest, arches))

        return rpms


def getNEVR(build):
    '''Extract RPM version identifier in NEVR format from Koji build object

    :param dict build: Koji buildinfo dictionary (as returned e.g. from
                       :meth:`koji.getBuild`)
    :return: NEVR string; epoch is included when non-zero, otherwise omitted
    :raise TaskotronValueError: if ``build`` is of incorrect type
    '''
    # validate input
    if (not isinstance(build, collections.Mapping) or 'nvr' not in build or
        'epoch' not in build):
        raise exc.TaskotronValueError("Input argument doesn't look like "
        "a Koji build object: %s" % build)

    rpmver = hawkey.split_nevra(build['nvr'] + '.noarch')
    rpmver.epoch = build['epoch'] or 0
    nevr = '%s-%s' % (rpmver.name, rpmver.evr())
    # supress 0 epoch (evr() method always includes epoch, even if 0)
    nevr = rpm_utils.rpmformat(nevr, fmt='nevr')

    return nevr
