# -*- coding: utf-8 -*-
# Copyright 2009-2014, Red Hat, Inc.
# License: GPL-2.0+ <http://spdx.org/licenses/GPL-2.0+>
# See the LICENSE file for more details on Licensing

from __future__ import absolute_import
import sys
import logging
import logging.handlers
import traceback


# http://docs.python.org/2/howto/logging.html#configuring-logging-for-a-library
log = logging.getLogger('libtaskotron')
log.addHandler(logging.NullHandler())


def _log_excepthook(*exc_info):
    '''Called when an exception is not caught'''
    log.critical(''.join(traceback.format_exception(*exc_info)))


def init(name='libtaskotron', level=logging.INFO, stream=True, syslog=False,
         filelog=None, set_rootlogger=False):
    """Setup a logger

    :param str name: name of the logger, 'libtaskotron' is default
    :param level: level of logging
    :type level: level definitions from :mod:`logging`
    :param bool stream: enables logging to stderr
    :param bool syslog: enables logging to syslog
    :param str filelog: enables logging to a file, the value is the file name
    :param bool set_rootlogger: adds output handlers to root logger
    """

    # overwrite default except hook
    sys.excepthook = _log_excepthook

    logger = logging.getLogger(name)
    rootlogger = logging.getLogger()
    logger.setLevel(level)
    logger.addHandler(logging.NullHandler())

    fmt = '[%(name)s:%(filename)s:%(lineno)d] '\
          '%(asctime)s %(levelname)-7s %(message)s'
    datefmt = '%Y-%m-%d %H:%M:%S'
    formatter = logging.Formatter(fmt=fmt, datefmt=datefmt)

    if stream:
        stream_handler = logging.StreamHandler()
        stream_handler.setFormatter(formatter)
        if set_rootlogger:
            rootlogger.addHandler(stream_handler)
        logger.debug("doing stream logging")

    if syslog:
        syslog_handler = logging.handlers.SysLogHandler(address='/dev/log',
                            facility=logging.handlers.SysLogHandler.LOG_LOCAL4)
        syslog_handler.setFormatter(formatter)
        if set_rootlogger:
            rootlogger.addHandler(syslog_handler)
        logger.debug("doing syslog logging")

    if filelog:
        file_handler = logging.handlers.RotatingFileHandler(filelog,
            maxBytes=500000, backupCount=5)
        file_handler.setFormatter(formatter)
        if set_rootlogger:
            rootlogger.addHandler(file_handler)
        logger.debug('doing file logging to %s', filelog)

    return logger
